;*****************************************************************************
; Example of ARM mode assembly source file that can be buit with v2.x and v4.x 
; TMS470 compilers and under either TIABI or TI_ARM9_ABI
; This file shows assembly source equivalent of this C routine:
;
;	int max(int a, int b)
;	{
;		return a < b ? b : a;
;	}
;
; The main body of the max routine is in ARM mode 
;*****************************************************************************


;*****************************************************************************
;* Define your own variables for which ABI is being used
; __TI_ARM9ABI_ASSEMBLER is a predefined symbol in v4.1.x code generation tools
;*****************************************************************************
        .if $$isdefed("__TI_ARM9ABI_ASSEMBLER") ; Use predefined symbol
ARM9ABI .set    __TI_ARM9ABI_ASSEMBLER		; if it exists
        .else					; otherwise
ARM9ABI .set    0				; cannot be using TI_ARM9_ABI
        .endif

	.if ARM9ABI				; TIABI is opposite of ARM9ABI
TIABI	.set	0
	.else
TIABI	.set	1
	.endif


;*****************************************************************************
;* TIABI requires a veneer for changing mode from Thumb to ARM
;*****************************************************************************
	.if	TIABI				
	.sect	".text:v$0"			; sub-section for veneer code
	.clink					; tells linker to remove this
						; section if it is not used
	.global	$max				
	.state16				; Start of Thumb mode routine

$max:
	BX        pc				;veneer code
	NOP
	.state32
	B         _max				; Branch to ARM mode routine
	.endif					

;*****************************************************************************
; Common directives for ARM mode function in any ABI
;*****************************************************************************
	.sect	".text"
	.state32
	.clink
	.global	_max

;*****************************************************************************
; Additional directive for ARM9ABI
;*****************************************************************************
	.if	ARM9ABI
	.armfunc  _max				; Marks state of function as ARM
	.endif

;*****************************************************************************
; Main body of ARM function
;*****************************************************************************
_max:
        CMP       A1, A2
        MOVLT     A1, A2
        BX        LR

