;*****************************************************************************
; Example of Thumb mode assembly source file that can be buit with v2.x and 
; v4.x TMS470 compilers and under either TIABI or TI_ARM9_ABI
; This file shows assembly source equivalent of this C routine:
;
;	extern int total;
;	sum(int val1, int val2)
;	{
;		int val = max(val1, val2);
;		total += val;
;	}
;
; Function sum makes a call to max, which is defined in file arm_ex.asm
; The main body of the sum routine is in Thumb mode 
;*****************************************************************************


;*****************************************************************************
;* Define your own variables for which ABI is being used
;*****************************************************************************
        .if $$isdefed("__TI_ARM9ABI_ASSEMBLER") ; Use predefined symbol
ARM9ABI .set    __TI_ARM9ABI_ASSEMBLER		; if it exists
        .else					; otherwise
ARM9ABI .set    0				; cannot be using TI_ARM9_ABI
        .endif

	.if ARM9ABI				; TIABI is opposite of ARM9ABI
TIABI	.set	0
	.else
TIABI	.set	1
	.endif

	.global	_sum

;*****************************************************************************
;* TIABI requires a veneer for changing mode from ARM to Thumb, and back
;*****************************************************************************
	.if	TIABI
	.sect	".text:v$0"			; sub-section for veneer code
	.clink					; tells linker to remove this
						; section if it is not used
	.state32				; Start of ARM mode

_sum:
	STMFD     sp!, {lr}
	ADD       lr, pc, #1
	BX        lr
	.state16
	BL        $sum
	BX        pc
	NOP
	.state32
	LDMFD     sp!, {pc}
	.endif			; .if TIABI

;*****************************************************************************
; Common directives for thumb mode function in any ABI
;*****************************************************************************
	.state16
	.sect	".text"
	.clink

;*****************************************************************************
; Different entry points for sum depending on ABI
;*****************************************************************************
	.if	ARM9ABI
	.thumbfunc	_sum		; Marks state of function as Thumb
_sum:	
	.else	; TIABI
	.global	$sum
$sum:
	.endif

;*****************************************************************************
; Main body of sum in thumb mode
;*****************************************************************************
        PUSH      {LR}

;*****************************************************************************
; Function calls are different depending on ABI
;*****************************************************************************
	.if	ARM9ABI			; if ARM9ABI
	.global   _max
	BL        _max			; function names always begin with "_" 
	.else	; TIABI			; otherwise, TIABI
	.global   $max
        BL        $max 			; Thumb functions begin with "$"
	.endif

        LDR       A2, CON1
        LDR       A3, [A2, #0]
        ADD       A1, A1, A3
        STR       A1, [A2, #0]

	POP	{A3}
	BX	A3

;*****************************************************************************
;* Constant table
;*****************************************************************************
	.sect	".text"
	.align	4
	.global		_total
CON1:	.field  	_total,32

